/***********************************************************

    hbs reduction subroutine: hbs2ossub.c

                                   2000/02/05  K. Kawabata

    Subtraction of second-order spectrum

    usage: % hbs2ossub infn reffile outfn

      infn -- input spectrum (FITS format)
      reffile -- 2nd order spectrum data
      outfn -- output spectrum (FITS format)

************************************************************/

#include<stdio.h>
#include<string.h>
#include<math.h>
#include<stdlib.h>
#include<unistd.h>
#include"fitsio.h"
#include"hbsinterpol.c"

#include"hbsdefine.h"

#ifndef SSW
#define SSW 8000.00
#endif

#ifndef SEW
#define SEW 9400.00
#endif

int scanfile( char *infile, float count[]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 || naxis2 != 1 ){
    printf( "error: definition of NX2 or NY(=1) in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count, &anynul, &status ) != 0 ){
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }
  return( 0 );
}

int writefile( char *outfile, float count[] ){

  fitsfile *fp;
  int i, status;
  char buf[100];
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, outfile, READWRITE, &status ) != 0 ){
    printf( "Cannot open file '%s' with READWRITE mode. status=%d\n",\
            outfile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 ){
    printf( "Error: definition of NX2 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  if( naxis2 != 1 ){
    printf( "Error: definition of y-dimension in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_write_img( fp, TFLOAT, 1, npix,\
      count, &status ) != 0 ){
    printf( "Error in the function fits_write_img\n" );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }

  return( 0 );
}

int main( int argc, char *argv[] ){

  float c1[NX2], c2[NX2], x[NX2];

  char reffile[50];
  char infile[50];
  char outfile[50];

  FILE *fp;
  char buf[200], last_buf[200];
  int i;

  double lmd;
  double refspx[NX2], refspy[NX2];
  double c3;
  double c_ord1, c_ord2;
  int n_refsp;

/***** confirmation of input file *****/

  if( argc != 4 ){
    printf( "usage: hbs2ossub infn ref_fn outfn\n" );
    exit( 1 );
  }

  strcpy( infile, argv[1] );
  strcpy( reffile, argv[2] );
  strcpy( outfile, argv[3] );


/* read input file */

  if( scanfile( infile, c1 ) != 0 ){
    printf( "Read error in '%s'\n", infile );
    exit( -1 );
  }

  for( i=0; i<=(NX2-1); i+=1 ){
    x[i] = SW + (float)i * DW;
  }

/* read 2nd-order spectrum reference file */

  n_refsp = -1;
  if( ( fp = fopen( reffile, "rt" ) ) == NULL ){
    printf( "2nd-order spectrum table '%s' not found.\n", reffile );
    exit( -1 );
  }
  while( feof( fp ) == 0 ){
    fgets( buf, 200, fp );
    if( strcmp( buf, last_buf ) == 0 )
      continue;
    n_refsp++;
    if( n_refsp >= NX2 ){
      printf( "n_refsp exceeds NX2.\n" );
      break;
    }
    sscanf( buf, "%lf %lf", &refspx[n_refsp], &refspy[n_refsp] );
    strcpy( last_buf, buf );
  }
  fclose( fp );

/*** 2nd-order spectrum subtraction ***/

  for( i=0; i<=(NX2-1); i+=1 ){

    lmd = SW + (float)i * DW;

    if( lmd >= SSW && lmd <= SEW ){
      c_ord1 = interpol( n_refsp, refspx, refspy, ( lmd / 2.0 ), 1 );
      c_ord2 = interpol( n_refsp, refspx, refspy, lmd, 1 );
      c3 = interpolf( ( NX2 -1 ), x, c1, ( lmd / 2.0 ), 1 );
/*printf( "%7.2f %e %e %e  %e %e\n", lmd, c_ord1, c_ord2, c3, c1[i], c2[i] );*/
      c2[i] = c1[i] - ( c_ord2 / c_ord1 ) * c3;
    }
    else{
      c2[i] = c1[i];
    }

/* write output file */

    if( writefile( outfile, c2 ) != 0 ){
      printf( "write failed in '%s'\n", outfile );
      return( 1 );
    }
  }

  return( 0 );
}
