/***********************************************************

    hbs reduction subroutine: hbsconerrbin.c

          1999/06/14  K. Kawabata   Modifying hbscbin.c
          2005/09/08  M. Isogai     Andor CCD

    Make binning table in order that every bin may have a constant error

    usage: % hbsconerrbin infn consterr ourfn

      infn -- input spectrum (FITS format)
      consterr -- constant error (real)
      outfn -- output binning table

************************************************************/

#include<stdio.h>
#include<string.h>
#include<math.h>
#include<stdlib.h>
#include<unistd.h>
#include"fitsio.h"
/* #include"hbsinterpol.c" */

#include"hbsdefine.h"

char detect[50];

int scanfile( char *infile, float count[]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 || naxis2 != 1 ){
    printf( "error: definition of NX2 or NY(=1) in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count, &anynul, &status ) != 0 ){
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }
  return( 0 );
}

int readheads( char *infile, char *keyword  ){

  fitsfile *fpf;
  int status;
  char comment[100];

  status = 0;

  if( fits_open_file( &fpf, infile, READONLY, &status ) != 0 ){
    return( -1 );
  }
  if( fits_read_keyword( fpf, keyword, detect,\
      comment, &status ) != 0 ){
    return( -1 );
  }
  if( fits_close_file( fpf, &status ) != 0 ){
    return( status );
  }
  return( 0 );
}

int readheadi( char *infile, char *keyword  ){

  fitsfile *fpf;
  int status;
  int value=0;
  char comment[100];

  status = 0;

  if( fits_open_file( &fpf, infile, READONLY, &status ) != 0 ){
    return( -1 );
  }
  if( fits_read_key( fpf, TINT, keyword, &value,\
      comment, &status ) != 0 ){
    return( -1 );
  }
  if( fits_close_file( fpf, &status ) != 0 ){
    return( status );
  }
  return( value );
}

int main( int argc, char *argv[] ){

  float c1[NX2];

  char infile[50];
  char outfile[50];
  char refout[50];
  char detector[50];

  int pregain, postgain;
  double adufactor; /* ADU per one electron */
  double consterr, c_thresh;

  int i, pix1, pix2;
  double lmdpix, widpix;
  double call,tmp;

  FILE *fp;

/***** confirmation of input file *****/

  if( argc != 4 ){
    printf( "usage: hbsconerrbin infn consterr outfn\n" );
    exit( 1 );
  }

  strcpy( infile, argv[1] );
  consterr = atof( argv[2] );
  strcpy( outfile, argv[3] );

/* read input file */

  if( scanfile( infile, c1 ) != 0 ){
    printf( "Read error in '%s'\n", infile );
    exit( 1 );
  }

/** read FITS HEADER ***/
  
  tmp = readheads( infile, "DETECTOR" );
  strcpy(detector,detect);
  if (strcmp(detector,"'1024x1024 TI CCD'") == 0 || strcmp(detector,"'T5 CRYOCAM (TC215)'") == 0 || strcmp(detector,"'S5 CRYOCAM (SITe)'") == 0)
    {
      pregain = readheadi( infile, "PREGAIN" );
      if( pregain == -1 ){
	printf( "Cannot find paramater 'PREGAIN' in %s\n", infile );
	exit( -1 );
      }
      postgain = readheadi( infile, "POSTGAIN" );
      if( postgain == -1 ){
	printf( "Cannot find paramater 'POSTGAIN' in %s\n", infile );
	exit( -1 );
      }
    }
  else if (strcmp(detector,"'Andor DV440'") == 0 )
    {
      pregain=1;
      postgain=1;
    }
  else
    {
      printf("Warning! header:detector is %s\n",detector);
    }

  adufactor = GEADU * pregain * postgain;
  c_thresh = pow( ( 100.0 / consterr ) , 2.0 ) * adufactor;
  
/* open output table */

  if( ( fp = fopen( outfile, "wt" ) ) == NULL ){
    printf( "Cannot open '%s'.\n", outfile );
    exit( -1 );
  }
  fprintf( fp, "3500.0000   JU\n" );
  fprintf( fp, "3550.0000   JU3500\n" );
  fprintf( fp, "3650.0000   JU3600\n" );
  fprintf( fp, "4400.0000   JB\n" );
  fprintf( fp, "4400.0000   JB4000\n" );
  fprintf( fp, "4400.0000   JB4100\n" );
  fprintf( fp, "4400.0000   JB4300\n" );
  fprintf( fp, "5500.0000   JV\n" );
  fprintf( fp, "6500.0000   CR\n" );
  fprintf( fp, "8000.0000   CI\n" );

/*** binning ***/

  pix2 = (int)(RSW - SW + 0.000001) / DW - 1;
  
  for( i=0; i<(NX2-11); i++ ){
    call = 0.0;
    pix1 = pix2 + 1;

    if( ( pix1 * DW + SW ) > REW ){
      i = 10000;
    }
    else{
      pix2 = pix1;
      call = c1[pix2];
      while( call < c_thresh && ( ( pix2 + 1 ) * DW + SW ) < REW ){
	pix2++;
	call += c1[pix2];
      }
      lmdpix = ( pix1 + pix2 ) / 2.0 * DW + SW;
      widpix = ( pix2 - pix1 + 1 ) * DW;

      fprintf( fp, "%9.4f %11.6f\n", lmdpix, widpix );
    }
  }
  
  fclose( fp );

  return( 0 );
}
